@@ -26,6 +26,8 @@
 #include <fluent-bit/flb_sds.h>
 #include <fluent-bit/flb_uri.h>
 #include <fluent-bit/flb_upstream_conn.h>
+#include <fluent-bit/flb_net_dns.h>
+#include <ares.h>
 
 /* Network connection setup */
 struct flb_net_setup {
@@ -43,6 +45,9 @@ struct flb_net_setup {
 
     /* maximum of times a keepalive connection can be used */
     int keepalive_max_recycle;
+
+    /* dns mode : TCP or UDP */
+    char *dns_mode;
 };
 
 /* Defines a host service and it properties */
@@ -55,37 +60,56 @@ struct flb_net_host {
     struct flb_uri *uri;   /* Extra URI parameters */
 };
 
-/* Defines an async DNS lookup context and its result event */
-struct flb_dns_lookup_context;
-
-struct flb_dns_lookup_result_event {
-    struct mk_event event;
-    flb_pipefd_t ch_events[2];
-    struct flb_dns_lookup_context *parent;
-};
-
+/* Defines an async DNS lookup context */
 struct flb_dns_lookup_context {
-    struct mk_event response_event;                  /* c-ares socket event */
-    int ares_socket_created;
-    void *ares_channel;
-    int result_code;
-    int finished;
-    struct mk_event_loop *event_loop;
-    struct flb_coro *coroutine;
-    struct addrinfo *result;
+    struct mk_event              response_event;                  /* c-ares socket event */
+    int                          ares_socket_registered;
+    struct ares_socket_functions ares_socket_functions;
+    int                         *udp_timeout_detected;
+    int                          ares_socket_created;
+    int                          ares_socket_type;
+    void                        *ares_channel;
+    int                         *result_code;
+    struct mk_event_loop        *event_loop;
+    struct flb_coro             *coroutine;
+    struct flb_sched_timer      *udp_timer;
+    int                          finished;
+    int                          dropped;
+    struct flb_net_dns          *dns_ctx;
+    struct addrinfo            **result;
     /* result is a synthetized result, don't call freeaddrinfo on it */
+    struct mk_list               _head;
 };
 
+#define FLB_DNS_LOOKUP_CONTEXT_FOR_EVENT(event) \
+    ((struct flb_dns_lookup_context *) \
+        &((uint8_t *) event)[-offsetof(struct flb_dns_lookup_context, response_event)])
+
+#define FLB_DNS_USE_TCP 'T'
+#define FLB_DNS_USE_UDP 'U'
+
+#define FLB_ARES_SOCKET_TYPE_TCP 1
+#define FLB_ARES_SOCKET_TYPE_UDP 2
+
 #ifndef TCP_FASTOPEN
 #define TCP_FASTOPEN  23
 #endif
 
 /* General initialization of the networking layer */
-void flb_net_init();
+void flb_net_lib_init();
+void flb_net_ctx_init(struct flb_net_dns *dns_ctx);
+
+void flb_net_dns_ctx_init();
+struct flb_net_dns *flb_net_dns_ctx_get();
+void flb_net_dns_ctx_set(struct flb_net_dns *dns_ctx);
+
 /* Generic functions */
 void flb_net_setup_init(struct flb_net_setup *net);
 int flb_net_host_set(const char *plugin_name, struct flb_net_host *host, const char *address);
 
+/* DNS handling */
+void flb_net_dns_lookup_context_cleanup(struct flb_net_dns *dns_ctx);
+
 /* TCP options */
 int flb_net_socket_reset(flb_sockfd_t fd);
 int flb_net_socket_tcp_nodelay(flb_sockfd_t fd);
