@@ -94,6 +94,14 @@ struct cio_chunk *cio_chunk_open(struct cio_ctx *ctx, struct cio_stream *st,
     /* Adjust counter */
     cio_chunk_counter_total_add(ctx);
 
+    /* Link the chunk state to the proper stream list */
+    if (cio_chunk_is_up(ch) == CIO_TRUE) {
+        mk_list_add(&ch->_state_head, &st->chunks_up);
+    }
+    else {
+        mk_list_add(&ch->_state_head, &st->chunks_down);
+    }
+
     return ch;
 }
 
@@ -116,6 +124,7 @@ void cio_chunk_close(struct cio_chunk *ch, int delete)
     }
 
     mk_list_del(&ch->_head);
+    mk_list_del(&ch->_state_head);
     free(ch->name);
     free(ch);
 
@@ -311,7 +320,12 @@ int cio_chunk_lock(struct cio_chunk *ch)
     }
 
     ch->lock = CIO_TRUE;
-    return cio_chunk_sync(ch);
+
+    if (cio_chunk_is_up(ch) == CIO_TRUE) {
+        return cio_chunk_sync(ch);
+    }
+
+    return CIO_OK;
 }
 
 int cio_chunk_unlock(struct cio_chunk *ch)
@@ -443,13 +457,34 @@ int cio_chunk_is_file(struct cio_chunk *ch)
     return CIO_FALSE;
 }
 
+static inline void chunk_state_sync(struct cio_chunk *ch)
+{
+    struct cio_stream *st;
+
+    if (!ch) {
+        return;
+    }
+
+    mk_list_del(&ch->_state_head);
+    st = ch->st;
+    if (cio_chunk_is_up(ch) == CIO_TRUE) {
+        mk_list_add(&ch->_state_head, &st->chunks_up);
+    }
+    else {
+        mk_list_add(&ch->_state_head, &st->chunks_down);
+    }
+}
+
 int cio_chunk_down(struct cio_chunk *ch)
 {
+    int ret;
     int type;
 
     type = ch->st->type;
     if (type == CIO_STORE_FS) {
-        return cio_file_down(ch);
+        ret = cio_file_down(ch);
+        chunk_state_sync(ch);
+        return ret;
     }
 
     return CIO_OK;
@@ -457,11 +492,14 @@ int cio_chunk_down(struct cio_chunk *ch)
 
 int cio_chunk_up(struct cio_chunk *ch)
 {
+    int ret;
     int type;
 
     type = ch->st->type;
     if (type == CIO_STORE_FS) {
-        return cio_file_up(ch);
+        ret = cio_file_up(ch);
+        chunk_state_sync(ch);
+        return ret;
     }
 
     return CIO_OK;
@@ -469,11 +507,14 @@ int cio_chunk_up(struct cio_chunk *ch)
 
 int cio_chunk_up_force(struct cio_chunk *ch)
 {
+    int ret;
     int type;
 
     type = ch->st->type;
     if (type == CIO_STORE_FS) {
-        return cio_file_up_force(ch);
+        ret = cio_file_up_force(ch);
+        chunk_state_sync(ch);
+        return ret;
     }
 
     return CIO_OK;
