@@ -146,6 +146,10 @@ static inline int _mk_event_del(struct mk_event_ctx *ctx, struct mk_event *event
 {
     int ret;
 
+    if ((event->status & MK_EVENT_REGISTERED) == 0) {
+        return 0;
+    }
+
     ret = epoll_ctl(ctx->efd, EPOLL_CTL_DEL, event->fd, NULL);
     MK_TRACE("[FD %i] Epoll, remove from QUEUE_FD=%i, ret=%i",
              event->fd, ctx->efd, ret);
@@ -155,6 +159,8 @@ static inline int _mk_event_del(struct mk_event_ctx *ctx, struct mk_event *event
 #endif
     }
 
+    MK_EVENT_NEW(event);
+
     return ret;
 }
 
