@@ -87,7 +87,10 @@ struct flb_kube *flb_kube_conf_create(struct flb_filter_instance *ins,
     /* Get Kubernetes API server */
     url = flb_filter_get_property("kube_url", ins);
 
-    if (ctx->use_kubelet) {
+    if (ctx->use_tag_for_meta) {
+        ctx->api_https = FLB_FALSE;
+    }
+    else if (ctx->use_kubelet) {
         ctx->api_host = flb_strdup(FLB_KUBELET_HOST);
         ctx->api_port = ctx->kubelet_port;
         ctx->api_https = FLB_TRUE;
@@ -184,8 +187,10 @@ struct flb_kube *flb_kube_conf_create(struct flb_filter_instance *ins,
         }
     }
 
-    flb_plg_info(ctx->ins, "https=%i host=%s port=%i",
-                 ctx->api_https, ctx->api_host, ctx->api_port);
+    if (!ctx->use_tag_for_meta) {
+        flb_plg_info(ctx->ins, "https=%i host=%s port=%i",
+                     ctx->api_https, ctx->api_host, ctx->api_port);
+    }
     return ctx;
 }
 
