@@ -437,11 +437,13 @@ static int unittest_rdclock (void) {
 
 /**@}*/
 
+extern int unittest_string (void);
 extern int unittest_cgrp (void);
 #if WITH_SASL_SCRAM
 extern int unittest_scram (void);
 #endif
 extern int unittest_assignors (void);
+extern int unittest_map (void);
 
 int rd_unittest (void) {
         int fails = 0;
@@ -450,6 +452,8 @@ int rd_unittest (void) {
                 int (*call) (void);
         } unittests[] = {
                 { "sysqueue",   unittest_sysqueue },
+                { "string",     unittest_string },
+                { "map",        unittest_map },
                 { "rdbuf",      unittest_rdbuf },
                 { "rdvarint",   unittest_rdvarint },
                 { "crc32c",     unittest_crc32c },
@@ -502,7 +506,7 @@ int rd_unittest (void) {
         for (i = 0 ; unittests[i].name ; i++) {
                 int f;
 
-                if (match && strcmp(match, unittests[i].name))
+                if (match && !strstr(unittests[i].name, match))
                         continue;
 
                 f = unittests[i].call();
