@@ -813,6 +813,7 @@ static int cb_s3_init(struct flb_output_instance *ins,
     ctx->s3_client->flags = 0;
     ctx->s3_client->proxy = NULL;
     ctx->s3_client->s3_mode = S3_MODE_SIGNED_PAYLOAD;
+    ctx->s3_client->retry_requests = ctx->retry_requests;
 
     ctx->s3_client->upstream = flb_upstream_create(config, ctx->endpoint, 443,
                                                    FLB_IO_TLS, ctx->client_tls);
@@ -2263,6 +2264,16 @@ static struct flb_config_map config_map[] = {
     "the documentation."
     },
 
+    {
+     FLB_CONFIG_MAP_BOOL, "auto_retry_requests", "false",
+     0, FLB_TRUE, offsetof(struct flb_s3, retry_requests),
+     "Immediately retry failed requests to AWS services once. This option "
+     "does not affect the normal Fluent Bit retry mechanism with backoff. "
+     "Instead, it enables an immediate retry with no delay for networking "
+     "errors, which may help improve throughput when there are transient/random "
+     "networking issues."
+    },
+
     {
      FLB_CONFIG_MAP_BOOL, "use_put_object", "false",
      0, FLB_TRUE, offsetof(struct flb_s3, use_put_object),
