@@ -21,12 +21,12 @@
 #ifndef FLB_OUT_STACKDRIVER_H
 #define FLB_OUT_STACKDRIVER_H
 
-#include <fluent-bit/flb_info.h>
-#include <fluent-bit/flb_output.h>
+#include <fluent-bit/flb_output_plugin.h>
 #include <fluent-bit/flb_oauth2.h>
 #include <fluent-bit/flb_sds.h>
 #include <fluent-bit/flb_pthread.h>
 #include <fluent-bit/flb_regex.h>
+#include <fluent-bit/flb_metrics.h>
 
 /* refresh token every 50 minutes */
 #define FLB_STD_TOKEN_REFRESH 3000
@@ -151,6 +151,12 @@ struct flb_stackdriver {
     /* upstream context for metadata end-point */
     struct flb_upstream *metadata_u;
 
+#ifdef FLB_HAVE_METRICS
+    /* metrics */
+    struct cmt_counter *cmt_successful_requests;
+    struct cmt_counter *cmt_failed_requests;
+#endif
+
     /* plugin instance */
     struct flb_output_instance *ins;
 
